<?php
# This file is distributed under GNU GPLv1 and further versions.
# This file was written by Zdenek Styblik
# This file is part of weathermap4rrd-ng
#
# Zdenek Styblik
# e-mail: zdenek <dot> styblik <at> gmail <dot> com
#
# Last update: 2008-07-07
#
class index extends defaultLib {
	public $settings;
	public $scriptname;
	public $debug = 0;
	private $cfgfile;
	private $arrow_width;
	private $rolloverlib;
	private $staticout = 0; // static or dynamic output?

	function __construct() {
		$this->settings = new StdClass;
		$settings = $this->settings;
		$settings->config = new StdClass;
		$settings->xmlobject = new StdClass;
		$settings->layout = "layout.xsl"; // NULL for XML output
		$settings->import = new StdClass;
		$settings->debug = new StdClass;
		$this->scriptname = $this->getScriptname();
		$settings->xmlobject->scriptname = $this->scriptname;
		$this->auto_index = -1;
	}

	public function init() {
		$path = $this->getPath();
		$settings = $this->settings;
		$auto_index = -1;
		$xml = $this->settings->xmlobject;
		if (file_exists("{$path}lib/configfile.php")) {
			if (isset($_GET['config'])) {
				$this->cfgfile = $this->stripMe($_GET['config']);
			} else {
				$this->cfgfile = NULL;
			}
			require_once "{$path}lib/configfile.php";
			$configfile = new config_file($this->cfgfile);
			if ($configfile->readconfig() !== false) {
				$config = $configfile->getConfig();
				$xml->refresh = $config->refresh;
				$xml->image = "{$this->scriptname}?module=drawmap";
				$this->rolloverlib = $configfile->getRollOverLib();
				$this->arrow_width = $configfile->getArrowWidth();
				if (isset($config->links) && is_object($config->links) && isset($config->nodes) && is_object($config->nodes)) {
					$nodes = $config->nodes;
					require_once "{$path}lib/graphic.php";
					$graphic = new graphic();
					foreach ($config->links as $link) {
						if (isset($link->linkinfourl) || isset($link->linkoverlibgraph)) {
							if (!isset($xml->map) || !is_object($xml->map)) {
								$xml->map = new StdClass;
							}
							if (!isset($link->linkinfourl)) 
								$link->linkinfourl = NULL;
							if (!isset($link->linkoverlibgraph))
								$link->linkoverlibgraph = NULL;
							$auto_index++;
							$xml->map->entry[$auto_index] = new StdClass;
							$map = & $xml->map->entry[$auto_index];
							$map = $this->get_jslib_obj($link->name, $link->linkoverlibgraph, $link->linkinfourl);
							$map->shape = "poly";
							$coords = array();
							if (isset($link->internode)) {
								// internode count
								$inodecount = count($link->internode);
								if (isset($link->internode) && is_array($link->internode) && isset($nodes->{$link->from}->x) &&
								 	isset($nodes->{$link->from}->y) && isset($nodes->{$link->to}->x) && isset($nodes->{$link->to}->y)) {
									if ($inodecount%2 == 0) {
										switch ($link->arrow) {
											case 'circle':
												$coords[0] = $graphic->draw_rectangle_half_map($nodes->{$link->from}->x, $nodes->{$link->from}->y,
												 	$link->internode[1]->x, $link->internode[1]->y,
												 	$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
												$coords[1] = $graphic->draw_rectangle_half_map($nodes->{$link->to}->x, $nodes->{$link->to}->y,
													$link->internode[$inodecount]->x, $link->internode[$inodecount]->y, 
													$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
												break;
											case 'halfarrow':
												$coords[2] = $graphic->draw_arrow_half_map($nodes->{$link->from}->x, $nodes->{$link->from}->y,
												 	$link->internode[1]->x, $link->internode[1]->y,
												 	$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
												$coords[3] = $graphic->draw_arrow_half_map($nodes->{$link->to}->x, $nodes->{$link->to}->y,
												 	$link->internode[$inodecount]->x, $link->internode[$inodecount]->y,
												 	$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
												break;
											default:
												$coords[4] = $graphic->draw_arrow_map($nodes->{$link->from}->x, $nodes->{$link->from}->y,
												 	$link->internode[1]->x, $link->internode[1]->y,
												 	$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
												$coords[5] = $graphic->draw_arrow_map($nodes->{$link->to}->x, $nodes->{$link->to}->y,
												 	$link->internode[$inodecount]->x, $link->internode[$inodecount]->y,
												 	$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
												break;
										}	
										// map areas between nodes
										$nodea = ($inodecount/2) + 1;
										$nodeb = $inodecount/2;
										for ($i = 1; $i <= $inodecount; $i++) {
											if ($i+1 <= $inodecount) {
												if (($i == $nodea || $i == $nodeb) && ($i+1 == $nodeb || $i+1 == $nodea)) {
													# skip
												} else {
													switch ($link->arrow) {
														case 'circle':
															$coords[6] = $graphic->draw_rectangle_half_map($link->internode[$i]->x, $link->internode[$i]->y,
															 	$link->internode[$i+1]->x, $link->internode[$i+1]->y,
															 	$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
															break;
														case 'halfarrow':
															$coords[7] = $graphic->draw_arrow_half_map($link->internode[$i]->x, $link->internode[$i]->y,
															 	$link->internode[$i+1]->x, $link->internode[$i+1]->y,
															 	$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
															break;
														default:
															$coords[8] = $graphic->draw_arrow_map($link->internode[$i]->x, $link->internode[$i]->y,
															 	$link->internode[$i+1]->x, $link->internode[$i+1]->y,
															 	$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
															break;
													}
												}
											}
										} // for
										// map areas between $nodea and $nodeb
										switch ($link->arrow) {
											case "circle":
												$coords[9] = $graphic->draw_rectangle_half_map($link->internode[$nodea]->x, $link->internode[$nodea]->y, 
													$graphic->middle($link->internode[$nodea]->x, $link->internode[$nodeb]->x), 
													$graphic->middle($link->internode[$nodea]->y, $link->internode[$nodeb]->y),
													$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
												break;
											case 'halfarrow':
												$coords[10] = $graphic->draw_arrow_half_map($link->internode[$nodea]->x, $link->internode[$nodea]->y, 
													$graphic->middle($link->internode[$nodea]->x, $link->internode[$nodeb]->x), 
													$graphic->middle($link->internode[$nodea]->y, $link->internode[$nodeb]->y),
													$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
												break;
											default:
												$coords[11] = $graphic->draw_arrow_map($link->internode[$nodea]->x, $link->internode[$nodea]->y, 
													$graphic->middle($link->internode[$nodea]->x, $link->internode[$nodeb]->x), 
													$graphic->middle($link->internode[$nodea]->y, $link->internode[$nodeb]->y),
													$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
												break;
										}
									} else {
										if (count($link->internode) == 1) {
											$j = 1;
											$inodelast = 1;
											$from = $nodes->{$link->from};
											$to = $nodes->{$link->to};
										} else {
											$j = round(count($link->internode)/2, 0);
											$inodelast = count($link->internode);
											$from = $link->internode[$j-1];
											$to = $link->internode[$j+1];
											switch ($link->arrow) {
												case 'circle':
													$coords[12] = $graphic->draw_rectangle_half_map($nodes->{$link->from}->x, $nodes->{$link->from}->y,
													 	$link->internode[1]->x, $link->internode[1]->y,
														$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
													$coords[13] = $graphic->draw_rectangle_half_map($nodes->{$link->to}->x, $nodes->{$link->to}->y, 
														$link->internode[$inodelast]->x, $link->internode[$inodelast]->y, 
														$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
													break;
												case 'halfarrow':
													$coords[14] = $graphic->draw_arrow_half_map($nodes->{$link->from}->x, $nodes->{$link->from}->y,
													 	$link->internode[1]->x, $link->internode[1]->y,
														$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
													$coords[15] = $graphic->draw_arrow_half_map($nodes->{$link->to}->x, $nodes->{$link->to}->y, 
														$link->internode[$inodelast]->x, $link->internode[$inodelast]->y, 
														$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
													break;
												default:
													$coords[16] = $graphic->draw_arrow_map($nodes->{$link->from}->x, $nodes->{$link->from}->y,
													 	$link->internode[1]->x, $link->internode[1]->y,
														$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
													$coords[17] = $graphic->draw_arrow_map($nodes->{$link->to}->x, $nodes->{$link->to}->y, 
														$link->internode[$inodelast]->x, $link->internode[$inodelast]->y, 
														$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
													break;
											}
										} // if count(link->internode)
										switch ($link->arrow):
											case "circle":
												$coords[18] = $graphic->draw_rectangle_half_map($from->x, $from->y,
												 	$link->internode[$j]->x, $link->internode[$j]->y,
													$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
												$coords[19] = $graphic->draw_rectangle_half_map($to->x, $to->y,
												 	$link->internode[$j]->x, $link->internode[$j]->y,
													$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
												break;
											case 'halfarrow':
												$coords[20] = $graphic->draw_arrow_half_map($from->x, $from->y,
											 		$link->internode[$j]->x, $link->internode[$j]->y,
													$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
												$coords[21] = $graphic->draw_arrow_half_map($to->x, $to->y,
												 	$link->internode[$j]->x, $link->internode[$j]->y,
													$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
												break;
											default:
												$coords[22] = $graphic->draw_arrow_map($from->x, $from->y,
												 	$link->internode[$j]->x, $link->internode[$j]->y,
													$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
												$coords[23] = $graphic->draw_arrow_map($to->x, $to->y,
												 	$link->internode[$j]->x, $link->internode[$j]->y,
													$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
												break;
										endswitch;
										for ($i = 1; $i <= $inodelast; $i++) {
											if ($i+1 <= $inodelast) {
												if (($i == ($j-1) && ($i+1) == $j) || ($i == $j && ($i+1) == ($j+1))) {
													# skip
												} else {
													switch ($link->arrow) {
														case 'circle':
															$coords[24] = $graphic->draw_rectangle_half_map($link->internode[$i]->x, $link->internode[$i]->y,
															 	$link->internode[$i+1]->x, $link->internode[$i+1]->y,
																$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
															break;
														case 'halfarrow':
															$coords[25] = $graphic->draw_arrow_half_map($link->internode[$i]->x, $link->internode[$i]->y,
															 	$link->internode[$i+1]->x, $link->internode[$i+1]->y,
																$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
															break;
														default:
															$coords[26] = $graphic->draw_arrow_map($link->internode[$i]->x, $link->internode[$i]->y,
															 	$link->internode[$i+1]->x, $link->internode[$i+1]->y,
																$this->arrow_width, $link->name, $link->linkoverlibgraph, $link->linkinfourl);
															break;
													}
												}
											}
										} // for
									} // else count % 2
								} else {
									# do nothing; this should never happen
									$this->debugMsg("Unexpected error - index.php");
								} // if isset $link -> internode && is_array
							} else { // if isset $link->internode
								if (isset($nodes->{$link->to}->x) && isset($nodes->{$link->from}->y)) {
									# else simple arrows;
									# one function per arrow
									switch ($link->arrow):
										case "circle":
											$coords[1] = $graphic->draw_rectangle_half_map($nodes->{$link->to}->x, $nodes->{$link->to}->y,
												$graphic->middle($nodes->{$link->from}->x, $nodes->{$link->to}->x), $graphic->middle($nodes->{$link->from}->y, $nodes->{$link->to}->y),
												$this->arrow_width);
											$coords[2] = $graphic->draw_rectangle_half_map($nodes->{$link->from}->x, $nodes->{$link->from}->y,
												$graphic->middle($nodes->{$link->from}->x, $nodes->{$link->to}->x), $graphic->middle($nodes->{$link->from}->y, $nodes->{$link->to}->y),
												$this->arrow_width);
											break;
										case 'halfarrow':
											$coords[3] = $graphic->draw_arrow_half_map($nodes->{$link->to}->x, $nodes->{$link->to}->y,
										 		$nodes->{$link->from}->x, $nodes->{$link->from}->y, $this->arrow_width);
											$coords[4] = $graphic->draw_arrow_half_map($nodes->{$link->from}->x, $nodes->{$link->from}->y,
										 		$nodes->{$link->to}->x, $nodes->{$link->to}->y, $this->arrow_width);
											break;
										default:
											$coords[5] = $graphic->draw_arrow_half_map($nodes->{$link->to}->x, $nodes->{$link->to}->y,
												$graphic->middle($nodes->{$link->from}->x, $nodes->{$link->to}->x), $graphic->middle($nodes->{$link->from}->y, $nodes->{$link->to}->y),
												$this->arrow_width);
											$coords[6] = $graphic->draw_arrow_half_map($nodes->{$link->from}->x, $nodes->{$link->from}->y,
												$graphic->middle($nodes->{$link->from}->x, $nodes->{$link->to}->x), $graphic->middle($nodes->{$link->from}->y, $nodes->{$link->to}->y),
												$this->arrow_width);
											break;
									endswitch;
								} // if isset(nodes->{link->to}->x)
							} // if isset link->internode
							if (is_array($coords))
								$map->coords = implode(",", $coords);
						} // if isset($link->infourl)
					} // foreach ($config->links as $link)
				} // if isset($config->links)
				if ($this->staticout == 1 && isset($config->outputfile)) {
					/* image is being generated separately / somewhere outside.
					*require_once "{$path}modules/drawmap.php";
					*$drawmap = new drawmap(0);
					*#$drawmap->setDebug(1);
					*$drawmap->init();
					*if (isset($drawmap->settings->debug->entry) && is_array($drawmap->settings->debug->entry)) {
					*	foreach ($drawmap->settings->debug->entry as $entry) {
					*		$this->debugMsg($entry);
					*	}
					*}
					*unset($drawmap);
					*/
					$pos = strrpos($config->outputfile, "/");
				  if (is_bool($result) && !$pos) {
						$xml->image = $config->outputfile;
				  } else {
						$part = substr($config->outputfile, ($pos+1));
						$xml->image = $part;
				  }									
				} else {
					$xml->image = "{$this->scriptname}?module=drawmap";
				}
				/* note@20080204 10:33pm
				 * this is probably waste of resources, since we are
				 * about to create xhtml here and once again in dlib.php
				 * by creating output. it /may be/ could be tricked thru
				 * _destroy(), however i don't have time to test it righ
				 * now.
				 */
				if (isset($config->htmlfile)) {
					$foo = "<root>\n";
					$foo.= $this->ObjToXML($xml);
					$foo.= "</root>";
					$template = $this->xslPrepare($settings->import, "root", "{$path}template/layout/{$settings->layout}");
					$htmlout = $this->transform($foo, $template);
					while(1) {
						if (!$handle = fopen("{$config->htmlfile}", 'w')) {
							$this->debugMsg("Unable to open file '{$config->htmlfile}' for writing");
							break;
						}
						if (fwrite($handle, $htmlout) == false) {
							$this->debugMsg("Unable to write into file '{$config->htmlfile}';");
							break;
						}
						fclose($handle);
						break;
					}
				}
				if ($this->debug == 1) {
					$xml->debug = $settings->debug;
				}
			} else {
				throw new Exception ("Config file not found. Please, make sure weathermap has access to it.");
			}
		} else {
			throw new Exception ("Internal file is missing. Error #99.");
		}
	}

	# desc:
	#
	# $linkname: string;
	# $overlib: string;
	# $infourl: string;
	# @return: object;
	public function get_jslib_obj($linkname = "foo", $overlib = NULL, $infourl = NULL) {
		$obj = new StdClass;
		if ($this->rolloverlib == "overlib" && !is_null($overlib)) {
			switch ($overlib) {
				case '1':
				case 'linkgraph':
					$url = "http://" . $_SERVER["HTTP_HOST"] . dirname($_SERVER['PHP_SELF']) . "linkgraph.php?link=" . urlencode($linkname)
					 	. "&config=" . urlencode($this->cfgfile) . "&limit=1";
					if (($fp = @fopen($url, 'rb')) !== false) {
						$buffer = '';
						while (!feof($fp)) {
							$buffer.= fread($fp, 8192);
						}	
						fclose($fp);
						$buffer = str_replace("\"", "", $buffer);
						$buffer = str_replace("\n", "", $buffer);
						$obj->onmouseover = "<![CDATA[return overlib('{$buffer}', STICKY, DELAY, 250, CAPTION, '{$linkname}', MOUSEOFF);]]>";
						$obj->onmouseout = "return nd();";
					} else {
						$this->debugMsg("Unable to open: {$url};");
					}// if $fp == false
					break;
				case 'currentvalues':
					$url = "http://" . $_SERVER["HTTP_HOST"] . dirname($_SERVER['PHP_SELF']) . "/currentvalues.php?link=" . urlencode($linkname)
					 	. "&config=" . urlencode($this->cfgfile);
					if (($fp = @fopen($url, 'rb')) !== false) {
						$buffer = '';
						while (!feof($fp)) {
							$buffer.= fread($fp, 8192);
						}	
						fclose($fp);
						$buffer = str_replace("\"", "", $buffer);
						$buffer = str_replace("\n", "", $buffer);
						$obj->onmouseover = "<![CDATA[return overlib('{$buffer}', DELAY, 250, CAPTION, 'Link current values');]]>";
						$obj->onmouseout = "return nd();";
					} else {
						$this->debugMsg("Unable to open: {$url};");
					}
					break;
				default:
					# Section ok
					/*
						#$html_code=" onmouseover=\"return overlib('<img src=$overlib>',DELAY,250,CAPTION,'$caption');\" ";
						$fp=fopen ("$overlib", "r");
						$buffer.= fread($fp, 1024);
						fclose($fp);
						$buffer=str_replace("\"","",$buffer);
						$buffer=str_replace("\n","<br>",$buffer);
						$html_code=" onmouseover=\"return overlib('$buffer',DELAY,250,CAPTION,'');\" ";
						$html_code.=" onmouseout=\"return nd();\" ";
					*/
					#$weathermap4php= $_SERVER["HTTP_HOST"].dirname($_SERVER['REQUEST_URI'])."/overlib";
					if (($fp = @fopen($overlib, 'rb')) !== false) {
						$buffer = '';
						while (!feof($fp)) {
							$buffer.= fread($fp, 8192);
						}	
						fclose($fp);
						$buffer = str_replace("\"", "", $buffer);
						$buffer = str_replace("\n", "", $buffer);
						$obj->onmouseover ="<![CDATA[return overlib('{$buffer}', STICKY, DELAY, 250, CAPTION, '{$linkname}', MOUSEOFF);]]>";
						$obj->onmouseout = "return nd();";
						// For Crinago
						/*
						$html_code=" onmouseover=\"return overlib('<img src=$overlib>',DELAY,250,CAPTION,'$caption');\" ";
						$html_code.=" onmouseout=\"return nd();\" ";
						*/
					} else {
						$this->debugMsg("Unable to open: {$url};");
					}	// if $fp == false
					break;
				// cases end
			} // switch
		} // if rolloverlib == overlib
		// linkinfourl
		if (!is_null($infourl)) {
			If ($infourl == "globalgraph") {
				$obj->linkinfourl = "http://" . $_SERVER["HTTP_HOST"] . dirname($_SERVER['PHP_SELF']) . "/linkgraph.php?link="
					. urlencode($linkname) . "&#38;config=" . urlencode($this->cfgfile);
			} else {
				// replace ampersands. however, we can't replace everything,
				// so be carefull.
				$infourl = str_replace("&", "&#38;", $infourl);
				$obj->linkinfourl = $infourl;
			} // if infourl
		} // if is_null($infourl)
		return $obj;
	}

	public function debugMsg($text = "foo") {
		$this->auto_index++;
		$debug = $this->settings->debug;
		$debug->entry[$this->auto_index] = new StdClass;
		$debug->entry[$this->auto_index]->text = $text;
	}
}
?>
